# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::DataShare::Mgmt::V2018_11_01_preview
  #
  # Creates a Microsoft.DataShare management client.
  #
  class ShareSubscriptions
    include MsRestAzure

    #
    # Creates and initializes a new instance of the ShareSubscriptions class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [DataShareManagementClient] reference to the DataShareManagementClient
    attr_reader :client

    #
    # Get shareSubscription in an account.
    #
    # Get a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscription] operation results.
    #
    def get(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      response = get_async(resource_group_name, account_name, share_subscription_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get shareSubscription in an account.
    #
    # Get a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      get_async(resource_group_name, account_name, share_subscription_name, custom_headers:custom_headers).value!
    end

    #
    # Get shareSubscription in an account.
    #
    # Get a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscription.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Create shareSubscription in an account.
    #
    # Create a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription [ShareSubscription] create parameters for
    # shareSubscription
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscription] operation results.
    #
    def create(resource_group_name, account_name, share_subscription_name, share_subscription, custom_headers:nil)
      response = create_async(resource_group_name, account_name, share_subscription_name, share_subscription, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Create shareSubscription in an account.
    #
    # Create a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription [ShareSubscription] create parameters for
    # shareSubscription
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_with_http_info(resource_group_name, account_name, share_subscription_name, share_subscription, custom_headers:nil)
      create_async(resource_group_name, account_name, share_subscription_name, share_subscription, custom_headers:custom_headers).value!
    end

    #
    # Create shareSubscription in an account.
    #
    # Create a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription [ShareSubscription] create parameters for
    # shareSubscription
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_async(resource_group_name, account_name, share_subscription_name, share_subscription, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, 'share_subscription is nil' if share_subscription.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscription.mapper()
      request_content = @client.serialize(request_mapper,  share_subscription)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscription.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscription.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Delete shareSubscription in an account.
    #
    # Delete a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [OperationResponse] operation results.
    #
    def delete(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      response = delete_async(resource_group_name, account_name, share_subscription_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_async(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      # Send request
      promise = begin_delete_async(resource_group_name, account_name, share_subscription_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::OperationResponse.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Get source share synchronization settings for a shareSubscription.
    #
    # Get synchronization settings set on a share
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<SourceShareSynchronizationSetting>] operation results.
    #
    def list_source_share_synchronization_settings(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      first_page = list_source_share_synchronization_settings_as_lazy(resource_group_name, account_name, share_subscription_name, skip_token:skip_token, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Get source share synchronization settings for a shareSubscription.
    #
    # Get synchronization settings set on a share
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_source_share_synchronization_settings_with_http_info(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      list_source_share_synchronization_settings_async(resource_group_name, account_name, share_subscription_name, skip_token:skip_token, custom_headers:custom_headers).value!
    end

    #
    # Get source share synchronization settings for a shareSubscription.
    #
    # Get synchronization settings set on a share
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param skip_token [String] Continuation token
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_source_share_synchronization_settings_async(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}/listSourceShareSynchronizationSettings'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version,'$skipToken' => skip_token},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::SourceShareSynchronizationSettingList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List Synchronizations in a share subscription.
    #
    # List synchronizations of a share subscription
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<ShareSubscriptionSynchronization>] operation results.
    #
    def list_synchronizations(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      first_page = list_synchronizations_as_lazy(resource_group_name, account_name, share_subscription_name, skip_token:skip_token, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # List Synchronizations in a share subscription.
    #
    # List synchronizations of a share subscription
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_synchronizations_with_http_info(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      list_synchronizations_async(resource_group_name, account_name, share_subscription_name, skip_token:skip_token, custom_headers:custom_headers).value!
    end

    #
    # List Synchronizations in a share subscription.
    #
    # List synchronizations of a share subscription
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param skip_token [String] Continuation token
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_synchronizations_async(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}/listSynchronizations'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version,'$skipToken' => skip_token},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronizationList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List data set level details for a share subscription synchronization
    #
    # List synchronization details
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<SynchronizationDetails>] operation results.
    #
    def list_synchronization_details(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, skip_token:nil, custom_headers:nil)
      first_page = list_synchronization_details_as_lazy(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, skip_token:skip_token, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # List data set level details for a share subscription synchronization
    #
    # List synchronization details
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_synchronization_details_with_http_info(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, skip_token:nil, custom_headers:nil)
      list_synchronization_details_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, skip_token:skip_token, custom_headers:custom_headers).value!
    end

    #
    # List data set level details for a share subscription synchronization
    #
    # List synchronization details
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param skip_token [String] Continuation token
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_synchronization_details_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, skip_token:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, 'share_subscription_synchronization is nil' if share_subscription_synchronization.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
      request_content = @client.serialize(request_mapper,  share_subscription_synchronization)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}/listSynchronizationDetails'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version,'$skipToken' => skip_token},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::SynchronizationDetailsList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Initiate an asynchronous data share job
    #
    # Initiate a copy
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of share subscription
    # @param synchronize [Synchronize] Synchronize payload
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionSynchronization] operation results.
    #
    def synchronize_method(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:nil)
      response = synchronize_method_async(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of share subscription
    # @param synchronize [Synchronize] Synchronize payload
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def synchronize_method_async(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:nil)
      # Send request
      promise = begin_synchronize_method_async(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method, FinalStateVia::AZURE_ASYNC_OPERATION)
      end

      promise
    end

    #
    # Request cancellation of a data share snapshot
    #
    # Request to cancel a synchronization.
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionSynchronization] operation results.
    #
    def cancel_synchronization(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:nil)
      response = cancel_synchronization_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def cancel_synchronization_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:nil)
      # Send request
      promise = begin_cancel_synchronization_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method, FinalStateVia::AZURE_ASYNC_OPERATION)
      end

      promise
    end

    #
    # List of available share subscriptions under an account.
    #
    # List share subscriptions in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param skip_token [String] Continuation Token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<ShareSubscription>] operation results.
    #
    def list_by_account(resource_group_name, account_name, skip_token:nil, custom_headers:nil)
      first_page = list_by_account_as_lazy(resource_group_name, account_name, skip_token:skip_token, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # List of available share subscriptions under an account.
    #
    # List share subscriptions in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param skip_token [String] Continuation Token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_account_with_http_info(resource_group_name, account_name, skip_token:nil, custom_headers:nil)
      list_by_account_async(resource_group_name, account_name, skip_token:skip_token, custom_headers:custom_headers).value!
    end

    #
    # List of available share subscriptions under an account.
    #
    # List share subscriptions in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param skip_token [String] Continuation Token
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_account_async(resource_group_name, account_name, skip_token:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name},
          query_params: {'api-version' => @client.api_version,'$skipToken' => skip_token},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Delete shareSubscription in an account.
    #
    # Delete a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [OperationResponse] operation results.
    #
    def begin_delete(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      response = begin_delete_async(resource_group_name, account_name, share_subscription_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Delete shareSubscription in an account.
    #
    # Delete a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_with_http_info(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      begin_delete_async(resource_group_name, account_name, share_subscription_name, custom_headers:custom_headers).value!
    end

    #
    # Delete shareSubscription in an account.
    #
    # Delete a shareSubscription in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_async(resource_group_name, account_name, share_subscription_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202 || status_code == 204
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::OperationResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Initiate an asynchronous data share job
    #
    # Initiate a copy
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of share subscription
    # @param synchronize [Synchronize] Synchronize payload
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionSynchronization] operation results.
    #
    def begin_synchronize_method(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:nil)
      response = begin_synchronize_method_async(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Initiate an asynchronous data share job
    #
    # Initiate a copy
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of share subscription
    # @param synchronize [Synchronize] Synchronize payload
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_synchronize_method_with_http_info(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:nil)
      begin_synchronize_method_async(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:custom_headers).value!
    end

    #
    # Initiate an asynchronous data share job
    #
    # Initiate a copy
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of share subscription
    # @param synchronize [Synchronize] Synchronize payload
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_synchronize_method_async(resource_group_name, account_name, share_subscription_name, synchronize, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, 'synchronize is nil' if synchronize.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::Synchronize.mapper()
      request_content = @client.serialize(request_mapper,  synchronize)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}/Synchronize'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Request cancellation of a data share snapshot
    #
    # Request to cancel a synchronization.
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionSynchronization] operation results.
    #
    def begin_cancel_synchronization(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:nil)
      response = begin_cancel_synchronization_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Request cancellation of a data share snapshot
    #
    # Request to cancel a synchronization.
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_cancel_synchronization_with_http_info(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:nil)
      begin_cancel_synchronization_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:custom_headers).value!
    end

    #
    # Request cancellation of a data share snapshot
    #
    # Request to cancel a synchronization.
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_cancel_synchronization_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'account_name is nil' if account_name.nil?
      fail ArgumentError, 'share_subscription_name is nil' if share_subscription_name.nil?
      fail ArgumentError, 'share_subscription_synchronization is nil' if share_subscription_synchronization.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
      request_content = @client.serialize(request_mapper,  share_subscription_synchronization)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataShare/accounts/{accountName}/shareSubscriptions/{shareSubscriptionName}/cancelSynchronization'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'accountName' => account_name,'shareSubscriptionName' => share_subscription_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronization.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get source share synchronization settings for a shareSubscription.
    #
    # Get synchronization settings set on a share
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [SourceShareSynchronizationSettingList] operation results.
    #
    def list_source_share_synchronization_settings_next(next_page_link, custom_headers:nil)
      response = list_source_share_synchronization_settings_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get source share synchronization settings for a shareSubscription.
    #
    # Get synchronization settings set on a share
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_source_share_synchronization_settings_next_with_http_info(next_page_link, custom_headers:nil)
      list_source_share_synchronization_settings_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Get source share synchronization settings for a shareSubscription.
    #
    # Get synchronization settings set on a share
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_source_share_synchronization_settings_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::SourceShareSynchronizationSettingList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List Synchronizations in a share subscription.
    #
    # List synchronizations of a share subscription
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionSynchronizationList] operation results.
    #
    def list_synchronizations_next(next_page_link, custom_headers:nil)
      response = list_synchronizations_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # List Synchronizations in a share subscription.
    #
    # List synchronizations of a share subscription
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_synchronizations_next_with_http_info(next_page_link, custom_headers:nil)
      list_synchronizations_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # List Synchronizations in a share subscription.
    #
    # List synchronizations of a share subscription
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_synchronizations_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionSynchronizationList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List data set level details for a share subscription synchronization
    #
    # List synchronization details
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [SynchronizationDetailsList] operation results.
    #
    def list_synchronization_details_next(next_page_link, custom_headers:nil)
      response = list_synchronization_details_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # List data set level details for a share subscription synchronization
    #
    # List synchronization details
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_synchronization_details_next_with_http_info(next_page_link, custom_headers:nil)
      list_synchronization_details_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # List data set level details for a share subscription synchronization
    #
    # List synchronization details
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_synchronization_details_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::SynchronizationDetailsList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List of available share subscriptions under an account.
    #
    # List share subscriptions in an account
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionList] operation results.
    #
    def list_by_account_next(next_page_link, custom_headers:nil)
      response = list_by_account_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # List of available share subscriptions under an account.
    #
    # List share subscriptions in an account
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_account_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_account_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # List of available share subscriptions under an account.
    #
    # List share subscriptions in an account
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_account_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataShare::Mgmt::V2018_11_01_preview::Models::ShareSubscriptionList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get source share synchronization settings for a shareSubscription.
    #
    # Get synchronization settings set on a share
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the shareSubscription.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [SourceShareSynchronizationSettingList] which provide lazy access to
    # pages of the response.
    #
    def list_source_share_synchronization_settings_as_lazy(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      response = list_source_share_synchronization_settings_async(resource_group_name, account_name, share_subscription_name, skip_token:skip_token, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_source_share_synchronization_settings_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # List Synchronizations in a share subscription.
    #
    # List synchronizations of a share subscription
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionSynchronizationList] which provide lazy access to
    # pages of the response.
    #
    def list_synchronizations_as_lazy(resource_group_name, account_name, share_subscription_name, skip_token:nil, custom_headers:nil)
      response = list_synchronizations_async(resource_group_name, account_name, share_subscription_name, skip_token:skip_token, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_synchronizations_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # List data set level details for a share subscription synchronization
    #
    # List synchronization details
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param share_subscription_name [String] The name of the share subscription.
    # @param share_subscription_synchronization [ShareSubscriptionSynchronization]
    # Share Subscription Synchronization payload.
    # @param skip_token [String] Continuation token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [SynchronizationDetailsList] which provide lazy access to pages of
    # the response.
    #
    def list_synchronization_details_as_lazy(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, skip_token:nil, custom_headers:nil)
      response = list_synchronization_details_async(resource_group_name, account_name, share_subscription_name, share_subscription_synchronization, skip_token:skip_token, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_synchronization_details_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # List of available share subscriptions under an account.
    #
    # List share subscriptions in an account
    #
    # @param resource_group_name [String] The resource group name.
    # @param account_name [String] The name of the share account.
    # @param skip_token [String] Continuation Token
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ShareSubscriptionList] which provide lazy access to pages of the
    # response.
    #
    def list_by_account_as_lazy(resource_group_name, account_name, skip_token:nil, custom_headers:nil)
      response = list_by_account_async(resource_group_name, account_name, skip_token:skip_token, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_account_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
